Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Runtime.Serialization
Imports System.ServiceModel
Imports System.Text
Imports System.Globalization
Imports System.ServiceModel.Syndication
Imports System.Xml
Imports System.IO
Imports System.Web.UI
Imports System.ServiceModel.Web

Namespace MSDNMagazine
	Public Class MSDNMagazineServiceType
		Implements IMSDNMagazineService

		Public Function GetServiceDoc() As AtomPub10ServiceDocumentFormatter Implements IMSDNMagazineService.GetServiceDoc
			Dim ctx As OutgoingWebResponseContext = WebOperationContext.Current.OutgoingResponse
			ctx.ContentType = "application/atomsvc+xml"
			Dim ret As AtomPub10ServiceDocumentFormatter = Nothing
			'create the ServiceDocument type
			Dim doc As New ServiceDocument()
			Dim ictx As IncomingWebRequestContext = WebOperationContext.Current.IncomingRequest
			'set the BaseUri to the current request URI
			doc.BaseUri = ictx.UriTemplateMatch.RequestUri
			'create a Collection of resources   
			Dim resources As List(Of ResourceCollectionInfo) = New List(Of ResourceCollectionInfo)()
			'creat the Blog resource
			Dim mainBlog As New ResourceCollectionInfo("MSDNMagazine", New Uri("feed", UriKind.Relative))
			'add the Accepts for this resource
			'remember this is the default if no accepts if present
			mainBlog.Accepts.Add("application/atom+xml;type=entry")
			resources.Add(mainBlog)
			'create the Pictures resource
			Dim mainPictures As New ResourceCollectionInfo("Pictures", New Uri("pictures", UriKind.Relative))
			'add the Accepts for this resource
			mainPictures.Accepts.Add("image/png")
			mainPictures.Accepts.Add("image/jpeg")
			mainPictures.Accepts.Add("image/gif")
			resources.Add(mainPictures)
			'create the Workspace
			Dim main As New Workspace("Main", resources)
			'add the Workspace to the Service Document
			doc.Workspaces.Add(main)

			'get the formatter
			ret = TryCast(doc.GetFormatter(), AtomPub10ServiceDocumentFormatter)
			Return ret
		End Function

		Private Shared _firstYear As Integer = 2001
		Public Function GetFeedForYear(ByVal year As String) As SyndicationFeedFormatter
			Dim ret As SyndicationFeedFormatter = Nothing
			Dim myFeedData As New SyndicationFeed()
			myFeedData.Title = New TextSyndicationContent("MSDN Magazine feed")
			Dim issues As IssuesCollection = GetAllIssues()

			Dim sitem As SyndicationItem = Nothing
			Dim list As List(Of SyndicationItem) = New List(Of SyndicationItem)()
			myFeedData.Items = list
			For Each item In issues
				sitem = New SyndicationItem With {.Title = New TextSyndicationContent("MSDN Magazine: " & item), .Content = New TextSyndicationContent("Article highlights go here")}
				sitem.Links.Add(SyndicationLink.CreateAlternateLink(New Uri("feeds/" & item, UriKind.Relative)))
				list.Add(sitem)
			Next item
			ret = New Atom10FeedFormatter(myFeedData)
			Return ret
		End Function
		Public Function GetIssuesFeed(ByVal format As String) As SyndicationFeedFormatter Implements IMSDNMagazineService.GetIssuesFeed
			Dim ret As SyndicationFeedFormatter = Nothing
			Dim myFeedData As New SyndicationFeed()
			myFeedData.Title = New TextSyndicationContent("MSDN Magazine feed")
			Dim articles As Articles = GetAllArticles()

			Dim sitem As SyndicationItem = Nothing
			Dim list As List(Of SyndicationItem) = New List(Of SyndicationItem)()
			myFeedData.Items = list
			Dim altLink As SyndicationLink = Nothing
			For Each item In articles
				sitem = New SyndicationItem With {.Title = New TextSyndicationContent("MSDN Magazine Article: " & item.Title), .Content = New XmlSyndicationContent(GenerateContent(item)), .PublishDate = GetPublished(item), .LastUpdatedTime = GetUpdated(item)}
				altLink = MakeLinkForArticle(item)
				sitem.Links.Add(altLink)
				list.Add(sitem)
			Next item
			If format IsNot Nothing AndAlso format = "rss" Then
				ret = New Rss20FeedFormatter(myFeedData)
			Else
				ret = New Atom10FeedFormatter(myFeedData)
			End If
			Return ret
		End Function

		Private Function GetUpdated(ByVal item As Article) As DateTimeOffset
			Return GetPublished(item)
		End Function

		Private Function GetPublished(ByVal item As Article) As DateTimeOffset
			Dim dt As New DateTimeOffset(New DateTime(Int32.Parse(item.Year), Int32.Parse(item.Issue), 2))
			Return dt
		End Function

		Private Function MakeLinkForArticle(ByVal item As Article) As SyndicationLink
			Dim articleUri As New Uri(String.Format("{0}/{1}/{2}", item.Year, item.Issue, item.Id), UriKind.Relative)

			Return SyndicationLink.CreateAlternateLink(articleUri)
		End Function
		Private Function GetAllArticles() As Articles
			Return _articles
		End Function

		Private Function GenerateContent(ByVal item As Article) As XmlReader
			Dim ms As New MemoryStream()
			Dim sw As New StreamWriter(ms)
			Dim w As New HtmlTextWriter(sw)
			w.AddAttribute(HtmlTextWriterAttribute.Type, "xhtml")
			w.RenderBeginTag(HtmlTextWriterTag.Div)
			w.AddAttribute("xmlns", "http://www.w3.org/1999/xhtml")
			w.RenderBeginTag(HtmlTextWriterTag.Div)
			w.Write(item.Content)
			w.RenderEndTag()
			w.RenderEndTag()
			w.Flush()
			ms.Seek(0, SeekOrigin.Begin)
			Dim r As XmlReader = XmlReader.Create(ms)
			Return r
		End Function
		Public Function GetAllIssues() As IssuesCollection Implements IMSDNMagazineService.GetAllIssues
			Dim allIssues As New IssuesCollection()
			Dim years As Integer = DateTime.Now.AddYears(1).Year - _firstYear
			For i As Integer = 0 To years - 1
				allIssues.Add((_firstYear + i).ToString())
			Next i
			allIssues.Sort(Function(s1, s2) Int32.Parse(s2).CompareTo(Int32.Parse(s1)))
			Return allIssues

		End Function



		Public Function GetIssuesByYear(ByVal year As String) As IssuesData Implements IMSDNMagazineService.GetIssuesByYear
			Dim issue As IssuesData = New IssuesData With {.Year = year, .Issues = New MonthInfo()}
			Dim c As Calendar = New GregorianCalendar()
			Dim ci As CultureInfo = CultureInfo.CurrentCulture

			Dim mnths As Integer = c.GetMonthsInYear(Int32.Parse(year))
			For j As Integer = 0 To mnths - 1
				issue.Issues.Add(ci.DateTimeFormat.MonthNames(j))
			Next j
			Return issue

		End Function



		#Region "IIssues Members"


		Public Function GetIssue(ByVal year As String, ByVal issue As String) As Articles Implements IMSDNMagazineService.GetIssue
			Dim ret = _
				From a In _articles _
				Where a.Year = year AndAlso a.Issue = issue _
				Select a
			Dim temp As New Articles()
			temp.AddRange(ret)
			Return temp
		End Function
		Private Shared _articles As Articles = InitArticles()

		Private Shared Function InitArticles() As Articles
			Dim ret As New Articles()
			ret.Add(New Article With {.Title = "SAAS: Connect Enterprise Apps with Hosted BizTalk Services ", .Content = "In this article we introduce you to BizTalk Services, new technology that offers the Enterprise Service Bus features of BizTalk Server as a hosted service.", .Author = "Jon Flanders and Aaron Skonnard", .Id = "1", .Year = "2008", .Issue = "6"})


			ret.Add(New Article With {.Title = "Concurrency: Tools And Techniques to Identify Concurrency Issues", .Content = "Efficient parallel applications arent born by merely running an old app on a parallel processor machine. Tuning needs to be done if youre to gain maximum benefit.", .Author = "Rahul V. Patil and Boby George", .Id = "2", .Year = "2008", .Issue = "6"})

			ret.Add(New Article With {.Title = "Robotics: Simulating the World with Microsoft Robotics Studio", .Content = "Microsoft Robotics Studio is not just for playing with robots. It also allows you to build service-based applications for a wide range of hardware devices.", .Author = "Sara Morgan", .Id = "3", .Year = "2008", .Issue = "6"})

			ret.Add(New Article With {.Title = "Form Filler: Build Workflows to Capture Data and Create Documents", .Content = "Learn how to create a workflow that uses InfoPath forms and other office documents for passing data to targeted activities and for use in Office documents.", .Author = "Rick Spiewak", .Id = "4", .Year = "2008", .Issue = "6"})
			Return ret
		End Function

		Public Function GetArticle(ByVal year As String, ByVal issue As String, ByVal article As String) As Article Implements IMSDNMagazineService.GetArticle
			Dim a As Article = Nothing
			a = ( _
					From arts In _articles _
					Where arts.Id = article AndAlso arts.Issue = issue AndAlso arts.Year = year _
					Select arts).Single()
			Return a
		End Function

		#End Region

		#Region "IIssues Members"


		Public Function AddArticle(ByVal year As String, ByVal issue As String, ByVal article As Article) As Article Implements IMSDNMagazineService.AddArticle
			Dim count = ( _
					From a In _articles _
					Where a.Year = year AndAlso a.Issue = issue _
					Select a).Count()
			article.Id = (count + 1).ToString()
			_articles.Add(article)
			Return article

		End Function

		#End Region


	End Class
End Namespace
